import processing.core.*; 
import processing.data.*; 
import processing.opengl.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class play extends PApplet {


ArrayList<PlaySpace> spaces = new ArrayList<PlaySpace>();

public void setup(){
  size(500,750);
  spaces.add(new Cans(0,0));
  spaces.add(new Attention(1,0));
  spaces.add(new Gears(0,1));
  spaces.add(new Tris(1,1));
  spaces.add(new Spiral(0,2));
  spaces.add(new Break(1,2));
}
public void draw(){
 
  for(PlaySpace ps : spaces){
     ps.draw(); 
  }
}

PlaySpace currentSpace;

public void keyPressed(){
   if(currentSpace != null){
      currentSpace.keyPressed(key);
   } 
}


public void mousePressed(){
   PlaySpace newSpace = findSpace();
   if(newSpace == null) return;
   newSpace.mousePressed(mouseX,mouseY);
   currentSpace = newSpace;
}

public void mouseReleased(){
   PlaySpace newSpace = findSpace();
   if(newSpace == null) return;
   newSpace.mouseReleased(mouseX,mouseY);
   
}

public void mouseDragged(){
   PlaySpace newSpace = findSpace();
   if(newSpace == null) return;
   newSpace.mouseDragged(mouseX,mouseY);
 
}

public void mouseMoved(){
     PlaySpace newSpace = findSpace();
   if(newSpace == null) return;
   newSpace.mouseMoved(mouseX,mouseY);
}


public PlaySpace findSpace(){
   for(PlaySpace ps : spaces){
      if(ps.isIn(mouseX,mouseY)){
         return ps; 
      }
   } 
   return null;
}


class Pair{
  float x,y;
  Pair(float px,float py){
      x = px; y = py;
  }
}





int GRIDSIZE = 2;

abstract class PlaySpace{
  
  public boolean isIn(float mx, float my){
    if( mx >= x * w && mx < (x+1)* w){
       if(my >= y * h && my < (y+1)*h){
          return true;
          
       } 
    }
  return false;
  }
  int bgc;
  int x, y;
  float w,h;
   PlaySpace(int px, int py){
    bgc = color(random(32,128));
    x = px;
    y = py;
    w = width/2;
    h = height/3;
   } 
   public void draw(){
     pushMatrix();
     noStroke();
    translate(x*w,y*w);
    
      run();
    popMatrix();
   } 
  public abstract void run();
  public void mousePressed(float mx, float my){
    doMousePressed(mx - (x*w),my-(y*h));
  }
  public void doMousePressed(float x, float y){}

  public void mouseReleased(float mx, float my){
    doMouseReleased(mx - (x*w),my-(y*h));
  }
  public void doMouseReleased(float x, float y){}

  public void mouseDragged(float mx, float my){
    doMouseDragged(mx - (x*w),my-(y*h));
  }
  public void doMouseDragged(float x, float y){}
  
  public void mouseMoved(float mx, float my){
    doMouseMoved(mx - (x*w),my-(y*h));
  }
  public void doMouseMoved(float x, float y){}
  
  public void keyPressed(char k){
     if(k == ' '){
      reset();
     } 
  }
  public void reset(){
    
  }
  
}
class Attention extends PlaySpace{
  Attention(int x,int y){
     super(x,y); 
     
  }
  
  
  float mx, my;
  public void run(){
    fill(bgc);
      rect(0,0,w,h);
    stroke(0);
    strokeWeight(2);
    noStroke();
    fill(222);
    
    for(int x = 1; x < 10; x++){
       for(int y = 1; y < 10; y++){
        float cx = x * (w/10);
        float cy =  y * (h/10);
        float d =  dist(cx,cy,mx,my)/6;
          rect(cx - d/2, cy - d/2 , d,d);
       } 
    }
        
    
  }
  public void doMouseMoved(float x, float y){
     mx = x;
     my = y;
  }
}
class Break extends PlaySpace {

 

boolean clear = true;

  public void reset() {
    clear = true;
  }

  Break(int x, int y) {
    super(x, y);
  }
  public void run() {
      if(clear){
        noStroke();
      fill(255);
      rect(0,0,w,h);
      clear = false;
      
  }
    strokeWeight(1);
    stroke(0);
    if(doit)    {
     line(0,0,a,b); 
     line(w,0,a,b); 
     line(0,h,a,b); 
     line(w,h,a,b); 
    }
doit = false;
  }
  boolean doit;
  float a,b;
  public void doMousePressed(float mx, float my) {
     doit = true;
    a = mx;
    b = my;
  }
}


class Cans extends PlaySpace {

  ArrayList<GearThing> gearthings = new ArrayList<GearThing>();

  public void reset() {
    gearthings = new ArrayList<GearThing>();
  }

  Cans(int x, int y) {
    super(x, y);
  }
  public void run() {
    fill(bgc);
      rect(0,0,w,h);
    noStroke();
    for (GearThing gt : gearthings) {
      gt.move();
      gt.draw();
    }
  }
  public void doMousePressed(float mx, float my) {
    gearthings.add(new GearThing(this, mx, my));
  }
}


class GearThing {
  Pair place; 
  Pair speed;
  Cans parent;
  GearThing(Cans g, float px, float py) {
    parent  = g;
    place = new Pair(parent.w/8, py);
    speed = new Pair(0, 0);
    
  }
  public void move() {
    if (place.x < parent.w/2) {
      speed.x += .2f;
    } 
    else {
      speed.x -= .2f;
    }
    place.x += speed.x;
  }
  public void draw() {
    fill(222);
    noStroke();
    strokeWeight(2);
    ellipse(place.x, place.y, 10, 10);
  }
}

class Gears extends PlaySpace{
  Gears(int x,int y){
     super(x,y); 
     
     
     
  }
  
    public void reset() {
    
      currentGear = null;
      lastGear = null; 
       gears = new ArrayList<Gear>();
  }

  
  Gear currentGear;
  Gear lastGear;
  ArrayList<Gear> gears = new ArrayList<Gear>();
  public void run(){
    
      fill(bgc);
      rect(0,0,w,h);
        for(Gear g : gears){
           g.draw(); 
        }
        if(currentGear != null){
           currentGear.sz+= .3f; 
        }
    
  }
  public void doMousePressed(float x, float y){
    currentGear = new Gear(x,y,lastGear);
    gears.add(currentGear);
  }
  public void doMouseDragged(float x, float y){
    if(currentGear != null){
       currentGear.x = x;
      currentGear.y = y; 
    }
  }
  
public void doMouseReleased(float x, float y){
  lastGear = currentGear;
  currentGear = null;
}


}

class Gear{
  float x,y,sz,a;
  Gear p;
  Gear(float px, float py, Gear prevGear){
     x = px;
      y = py; 
      sz = 5;
      p = prevGear;
  };
  public void draw(){
    stroke(255);
    noFill();
     ellipse(x,y,sz,sz); 
     a += .1f;
     if(p != null){
        line(x + cos(a)*sz/2,y + sin(a)*sz/2
              ,p.x + cos(p.a)*p.sz/2,p.y + sin(p.a)*p.sz/2);
     }
     
  }
  
  
  
}
class Spiral extends PlaySpace {

  ArrayList<Lin> gearthings = new ArrayList<Lin>();

boolean clear = true;

  public void reset() {
    gearthings = new ArrayList<Lin>();

    clear = true;
  }

  Spiral(int x, int y) {
    super(x, y);
  }
  public void run() {
if(clear){
        noStroke();
    fill(222);
    rect(0,0,w,h);
    clear = false;
}
    noStroke();
    ArrayList<Lin> linToKill = new ArrayList<Lin>();
    for (Lin gt : gearthings) {
      if(!gt.move()){
        linToKill.add(gt);
      } else gt.draw();
    }
    
    gearthings.removeAll(linToKill);
  }
  public void doMousePressed(float mx, float my) {
    gearthings.add(new Lin(this, mx, my));
  }
}


class Lin {
float speed ;
  Spiral parent;
  int c;
  Lin(Spiral g, float px, float py) {
    parent  = g;
    speed = dist( g.w/2,g.h/2,px,py)/100.0f;

    c = color(random(0,255));

    x = lastx = g.w/2;
    y = lasty = g.h/2;

  }
  float lastx,lasty,x,y, a,d;
  public boolean move() {
    a += speed;
    d += speed;
    lastx = x;
    lasty = y;
    x = parent.w/2+ cos(a)*d;
    y = parent.h/2 + sin(a)*d;
    
    if(dist(parent.w/2,parent.h/2,x,y) > parent.w/2) {
      // println("DEATH"); 
      return false;
    }
    return true;
    
  }
  public void draw() {
    stroke(c);
    strokeWeight(3);
    line(lastx,lasty,x,y);
  }
}

class Tris extends PlaySpace {

  ArrayList<Tri> gearthings = new ArrayList<Tri>();

boolean clear = true;

  public void reset() {
    gearthings = new ArrayList<Tri>();

    clear = true;
  }

  Tris(int x, int y) {
    super(x, y);
  }
  public void run() {
if(clear){
        noStroke();
    fill(222);
    rect(0,0,w,h);
    clear = false;
}
    noStroke();
    for (Tri gt : gearthings) {
      gt.move();
      gt.draw();
    }
  }
  public void doMousePressed(float mx, float my) {
    gearthings.add(new Tri(this, mx, my));
  }
}


class Tri {
  Pair place; 
  Pair speed;
  Tris parent;
  Tri(Tris g, float px, float py) {
    parent  = g;
    place = new Pair(px, py);
    speed = new Pair(0, 0);
    
  }
  public void move() {
    if (place.x < parent.w/2) {
      speed.x += .2f;
    } 
    else {
      speed.x -= .2f;
    }
    place.x += speed.x;
    if (place.y < parent.h/2) {
      speed.y += .2f;
    } 
    else {
      speed.y -= .2f;
    }
    place.y += speed.y;
  }
  public void draw() {
    fill(255);
    stroke(0);
    strokeWeight(2);
    //triangle(place.x,place.y,place.x+10,place.y+10,place.x-10,place.y+10);
    
    ellipse(place.x,place.y,10,10);
      
    
  }
}

  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "play" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
